# -*- coding: utf-8 -*-

"""
/***************************************************************************
 ForstProcessing
                                 A QGIS plugin
 ForstProcessing
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2018-11-13
        copyright            : (C) 2018 by CHL
        email                : christina_ludwig@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

__author__ = 'CHL'
__date__ = '2018-11-13'
__copyright__ = '(C) 2018 by CHL'

# This will get replaced with a git SHA1 when you do a git archive

__revision__ = '$Format:%H$'

import os
from qgis.PyQt.QtCore import QCoreApplication, QVariant, QSettings, QFileInfo, Qt, QFile
from qgis.PyQt.QtWidgets import QMessageBox, QApplication, QFileDialog, QDialog
from qgis.PyQt.QtGui import QIcon, QFont
from qgis.core import (QgsProcessing, QgsVectorLayer, QgsField, QgsSettings, QgsVectorFileWriter, QgsWkbTypes,
                       QgsCoordinateReferenceSystem, QgsPointXY, QgsPoint, QgsFields,
                       QgsProcessingAlgorithm, QgsProject,
                       QgsProcessingParameterString, QgsTextBufferSettings,
                       QgsProcessingParameterFeatureSink, QgsFeature, QgsGeometry, QgsPalLayerSettings, QgsTextFormat,
                       QgsVectorLayerSimpleLabeling)
from qgis.gui import QgsEncodingFileDialog
from .generatecruisepoints_dialog import GenerateCruisePointsDialog
from .help_dialog import HelpDialog
from processing.gui.MessageBarProgress import MessageBarProgress
from processing.tools import dataobjects
from qgis.utils import iface

from ..resources import *
from .utils import random_points_within
import processing

# todo dialog verschwindet wenn ausgabedatei ausgewählt die schon existiert
# todo enable different outfile formats
# todo sort window modality
class GenerateCruisePoints(QgsProcessingAlgorithm):
    """
    This is an example algorithm that takes a vector layer and
    creates a new identical one.

    It is meant to be used as an example of how to create your own
    algorithms and explain methods and variables used to do it. An
    algorithm like this will be available in all elements, and there
    is not need for additional work.

    All Processing algorithms should extend the QgsProcessingAlgorithm
    class.
    """

    # Constants used to refer to parameters and outputs. They will be
    # used when calling the algorithm from another algorithm, or when
    # calling from the QGIS console.

    ADDRESS = ""
    OUTPUT = ""
    parent = None
    layer = None
    encoding = None
    n_points = 0

    def initAlgorithm(self, config):
        """
        Here we define the inputs and output of the algorithm, along
        with some other properties.
        """
        self.parent = None
        self.canvas = iface.mapCanvas()

        # We add the input vector features source. It can have any kind of
        # geometry.
        self.addParameter(
            QgsProcessingParameterString(
                self.ADDRESS,
                'Adresse'
            )
        )

        # We add a feature sink in which to store our processed features (this
        # usually takes the form of a newly created vector layer when the
        # algorithm is run in QGIS).
        self.addParameter(
            QgsProcessingParameterFeatureSink(
                self.OUTPUT,
                self.tr('Output layer')
            )
        )

        self.dlg = GenerateCruisePointsDialog()
        self.help_dlg = HelpDialog()

        # Connect signals and slots
        self.dlg.btnHelp.clicked.connect(self.help_dlg.show)
        self.dlg.toolOut.clicked.connect(self.get_output_file)
        self.dlg.buttonBox.rejected.connect(self.reject)
        self.dlg.buttonBox.accepted.connect(self.run)
        QgsProject().instance().layerRemoved.connect(self.add_layers_to_combobox)
        QgsProject().instance().layerWasAdded.connect(self.add_layers_to_combobox)

        self.add_layers_to_combobox()

    def run(self):
        if self.check_input() == 0:
            self.feedback = MessageBarProgress(algname=self.displayName())
            self.context = dataobjects.createContext(self.feedback)
            self.processAlgorithm(self.context, self.feedback)
            self.feedback.close()

    def processAlgorithm(self, context, feedback):
        """
        Here is where the processing itself takes place.
        """

        print("Start processing")
        if self.dlg.rdbRandom.isChecked():
            self.generate_random_points()
            point_layer = QgsVectorLayer(self.output_file, os.path.basename(self.output_file), "ogr")
        else:
            self.generate_point_grid()
            point_layer = QgsVectorLayer(self.output_file, os.path.basename(self.output_file).split(".")[0], "ogr")
            self.set_id_attribute(point_layer)

        QgsProject().instance().addMapLayer(point_layer)

        if self.dlg.chkLabel.isChecked():
            self.addLabels(point_layer)

        del feedback, context

        return {self.OUTPUT: "dest_id"}

    def name(self):
        """
        Returns the algorithm name, used for identifying the algorithm. This
        string should be fixed for the algorithm, and must not be localised.
        The name should be unique within each provider. Names should contain
        lowercase alphanumeric characters only and no spaces or other
        formatting characters.
        """
        return '3 - Punkteraster'

    def displayName(self):
        """
        Returns the translated algorithm name, which should be used for any
        user-visible display of the algorithm name.
        """
        return self.tr(self.name())

    def tr(self, string):
        return QCoreApplication.translate('Processing', string)

    def createInstance(self):
        return GenerateCruisePoints()

    def createCustomParametersWidget(self, parent):
        self.parent = parent
        return self.dlg

    def svgIconPath(self):
        return os.path.abspath("./icons/generate_cruisepoints.png")

    def icon(self):
        return QIcon(":/plugins/icons/generate_cruisepoints.png")

    def reject(self):
        self.dlg.close()

    def add_layers_to_combobox(self):
        """
        Add all polygon vector layers to the change box
        :return:
        """
        project = QgsProject().instance()
        layers = []
        for lay in project.mapLayers():
            if isinstance(project.mapLayer(lay), QgsVectorLayer):
                if project.mapLayer(lay).geometryType() == 2:
                    layers.append(project.mapLayer(lay).name())
        self.dlg.inShape.clear()
        self.dlg.inShape.addItems(layers)

    def check_input(self):
        """
        Check input parameters from GUI
        :return:
        """
        if self.dlg.inShape.currentText() == "":
            QMessageBox.warning(self.dlg, "Punkteraster", "Kein Eingabelayer angegeben.")
            return 1
        elif self.dlg.outShape.text() == "":
            QMessageBox.warning(self.dlg, "Punkteraster", "Kein Ausgabedatei angegeben.")
            return 1

        if os.path.isfile(self.output_file):
            try:
                os.unlink(self.output_file)
            except:
                print("error")
                QMessageBox.warning(self.dlg,
                                    "Punkteraster",
                                    "Ausgabedatei existiert bereits und konnte nicht gelöscht werden.")
                return 1

        self.layer = QgsProject().instance().mapLayersByName(self.dlg.inShape.currentText())[0]
        self.n_points = self.dlg.spnNumber.value()
        self.spacing_x = self.dlg.spnSpacingX.value()
        self.spacing_y = self.dlg.spnSpacingY.value()
        self.offset_x = self.dlg.spnInset.value()
        self.offset_y = self.dlg.spnInsetY.value()

        return 0

    def save_file_dialog(self):
        """
        Open file dialog so user can select where the file should be saved
        :return:
        """
        dirName = QSettings().value("/UI/lastShapefileDir")
        encode = QSettings().value("/UI/encoding")
        file_dialog = QgsEncodingFileDialog(self.dlg, "Shapefile speichern unter ...", dirName, encoding=encode)
        file_dialog.setDefaultSuffix("shp")
        file_dialog.setFileMode(QFileDialog.AnyFile)
        file_dialog.setAcceptMode(QFileDialog.AcceptSave)
        file_dialog.setOption(QFileDialog.DontConfirmOverwrite, False)
        if not file_dialog.exec_() == QDialog.Accepted:
            return None, None
        files = file_dialog.selectedFiles()
        QSettings().setValue("/UI/lastShapefileDir", QFileInfo(str(files[0])).absolutePath())
        return str(files[0]), str(file_dialog.encoding())

    def get_output_file(self):
        """
        Open a save file dialog and save the user given path to the output file
        :return:
        """
        self.output_file, self.encoding = self.save_file_dialog()
        if self.output_file is None or self.encoding is None:
            return 1
        self.dlg.outShape.setText(self.output_file)

    def create_file(self, fields):
        check = QFile(self.output_file)
        if check.exists():
            if not QgsVectorFileWriter.deleteShapeFile(self.output_file):
                QMessageBox.warning(self.dlg, "Warnung", "Datei existiert bereits und konnte nicht gelöscht werden.")
                return
        writer = QgsVectorFileWriter(self.output_file, self.encoding, fields, QgsWkbTypes.Point,
                                     self.layer.crs(), "ESRI Shapefile")
        return writer

    def write_features_to_file(self, points):
        """
        Write all points to a new vector file

        :param points:
        :return:
        """

        # fields
        fields = QgsFields()
        fields.append(QgsField("ID", QVariant.Int))
        fields.append(QgsField("x", QVariant.Double))
        fields.append(QgsField("y", QVariant.Double))

        writer = self.create_file(fields)

        feature = QgsFeature(fields)
        for idx, p in enumerate(points, 1):
            feature.setGeometry(p)
            feature.setAttribute(0, idx)
            feature.setAttribute(1, p.asPoint().x())
            feature.setAttribute(2, p.asPoint().y())
            writer.addFeature(feature)
        del writer
        return 0

    def generate_random_points(self):
        """
        Generate random points within a polygon
        :return:
        """

        if self.dlg.useSelected.checkState() == Qt.Checked:
            features = self.layer.getSelectedFeatures()
        else:
            features = self.layer.getFeatures()

        points = self.create_points(features, self.n_points)
        self.write_features_to_file(points)

    def create_points(self, features, n_points):
        """
        Creates n points for each polygon in features
        :param features:
        :param n_points:
        :return:
        """
        all_points = []
        for feat in features:
            points = random_points_within(feat.geometry(), n_points)
            all_points = all_points + points
        return all_points

    def regularize(self):

        # Create temporary memory layer
        settingTemp = "Point?crs=" + str(
            self.layer.crs().authid()) + "&field=ID:integer&field=RECHTSWERT:real(15,4)&field=HOCHWERT:real(15,4)"
        grid_vector = QgsVectorLayer(settingTemp, "Temporary Layer", "memory")

        # fields = temp.pendingFields()
        # outFeat = QgsFeature(fields)
        grid_vector.startEditing()

        # Get bounding box
        if self.dlg.useSelected.checkState() == Qt.Checked:
            bbox = self.layer.boundingBoxOfSelected()
        else:
            bbox = self.layer.extent()

        #id = 0
        y = bbox.yMinimum() + self.offset_y
        while y <= bbox.yMaximum():
            x = bbox.xMinimum() + self.offset_x
            while x <= bbox.xMaximum():
                pGeom = QgsGeometry().fromPointXY(QgsPointXY(x, y))
                fields = QgsFields()
                fields.append(QgsField("ID", QVariant.Int))
                fields.append(QgsField("x", QVariant.Double))
                fields.append(QgsField("y", QVariant.Double))
                new_point = QgsFeature(fields)
                new_point.setGeometry(pGeom)
                #new_point.setAttribute(0, id)
                new_point.setAttribute(1, x)
                new_point.setAttribute(2, y)
                grid_vector.addFeature(new_point)
                #id += 1
                x = x + self.spacing_x
            y = y + self.spacing_y

        # delete writer
        grid_vector.commitChanges()
        return grid_vector

    def generate_point_grid(self):
        self.grid_vector = self.regularize()
        result = processing.run("native:clip",
                                {'INPUT': self.grid_vector,
                                 'OUTPUT': self.output_file,
                                 'OVERLAY': self.layer},
                                context=self.context,
                                feedback=self.feedback)
        return result

    def set_id_attribute(self, layer):
        """
        Assigns an id to each point
        :param layer:
        :return:
        """
        layer.startEditing()
        for idx, feat in enumerate(layer.getFeatures(), 1):
            layer.changeAttributeValue(feat.id(), 0, idx)
        layer.commitChanges()

    def addLabels(self, layer):
        """
        Adds labels to the points
        :param layer: vector layer with points
        :return:
        """
        layer_pal_settings = QgsPalLayerSettings()
        text_format = QgsTextFormat()
        font = QFont("Arial", 12)
        font.setBold(True)
        text_format.setFont(font)
        text_format.setSize(12)
        layer_pal_settings.setFormat(text_format)

        layer_pal_settings.fieldName = "id"
        layer_pal_settings.placement = 2
        #layer_pal_settings.xOffset = -15
        #layer_pal_settings.yOffset = -3
        layer_pal_settings.enabled = True

        layer_settings = QgsVectorLayerSimpleLabeling(layer_pal_settings)
        layer.setLabelsEnabled(True)
        layer.setLabeling(layer_settings)
        layer.triggerRepaint()

        self.canvas.refresh()

