# -*- coding: utf-8 -*-

"""
/***************************************************************************
 ForstProcessing
                                 A QGIS plugin
 ForstProcessing
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2018-11-13
        copyright            : (C) 2018 by CHL
        email                : christina_ludwig@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

__author__ = 'CHL'
__date__ = '2018-11-13'
__copyright__ = '(C) 2018 by CHL'

# This will get replaced with a git SHA1 when you do a git archive

__revision__ = '$Format:%H$'

import os
from PyQt5.QtCore import QCoreApplication, QVariant
from PyQt5.QtWidgets import QMessageBox
from qgis.PyQt.QtGui import QIcon, QFont
from qgis.core import (QgsProcessing, QgsVectorLayer, QgsField, QgsSettings, QgsProject,QgsApplication,
                       QgsCoordinateReferenceSystem, QgsPointXY, QgsPoint,
                       QgsProcessingAlgorithm, QgsRasterLayer,
                       QgsProcessingParameterString,
                       QgsProcessingParameterFeatureSink, QgsFeature, QgsGeometry, QgsPalLayerSettings, QgsTextFormat,
                       QgsVectorLayerSimpleLabeling)
from processing.gui.MessageBarProgress import MessageBarProgress
from processing.tools import dataobjects
from qgis.utils import iface
import urllib3
import certifi
import ssl
import subprocess
from .reset_dialog import ResetDialog

import sys
from distutils.dir_util import copy_tree
from distutils.file_util import copy_file
from ..resources import *
from .utils import *


class ResetForstSettings(QgsProcessingAlgorithm):
    """
    This is an example algorithm that takes a vector layer and
    creates a new identical one.

    It is meant to be used as an example of how to create your own
    algorithms and explain methods and variables used to do it. An
    algorithm like this will be available in all elements, and there
    is not need for additional work.

    All Processing algorithms should extend the QgsProcessingAlgorithm
    class.
    """

    # Constants used to refer to parameters and outputs. They will be
    # used when calling the algorithm from another algorithm, or when
    # calling from the QGIS console.

    ADDRESS = ""
    OUTPUT = ""
    parent = None

    def initAlgorithm(self, config):
        """
        Here we define the inputs and output of the algorithm, along
        with some other properties.
        """
        self.parent = None
        self.canvas = iface.mapCanvas()
        self.layerid = ''
        self.srs = QgsCoordinateReferenceSystem("EPSG:32633")
        self.defaultPoint = pointTo3857(QgsPointXY(407719.7, 5279324.9), self.srs)

        if QgsSettings().value("addressLookup/mode") is None:
            QgsSettings().setValue("addressLookup/mode", 1)

        # We add the input vector features source. It can have any kind of
        # geometry.
        self.addParameter(
            QgsProcessingParameterString(
                self.ADDRESS,
                'Adresse'
            )
        )

        # We add a feature sink in which to store our processed features (this
        # usually takes the form of a newly created vector layer when the
        # algorithm is run in QGIS).
        self.addParameter(
            QgsProcessingParameterFeatureSink(
                self.OUTPUT,
                self.tr('Output layer')
            )
        )

        self.dlg = ResetDialog()
        self.dlg.buttonBox.accepted.connect(self.run)
        self.dlg.buttonBox.rejected.connect(self.reject)
        QgsSettings().setValue("resetForstSettings/mode", 'no')


    def run(self):
        self.processAlgorithm()


    def processAlgorithm(self):
        self.dlg.close()
        QgsSettings().setValue("resetForstSettings/mode", 'yes')
        #res = self.copy_settings()
        #if res == 0:
        QMessageBox.information(iface.mainWindow(), "Information", u"Die Einstellung wurden erfolgreich zurückgesetzt. Starten Sie QGIS mit dem Profile 'QGIS Forst' neu, um die Änderungen zu aktivieren.")


    def name(self):
        """
        Returns the algorithm name, used for identifying the algorithm. This
        string should be fixed for the algorithm, and must not be localised.
        The name should be unique within each provider. Names should contain
        lowercase alphanumeric characters only and no spaces or other
        formatting characters.
        """
        return '7 - Standardeinstellungen'

    def displayName(self):
        """
        Returns the translated algorithm name, which should be used for any
        user-visible display of the algorithm name.
        """
        return self.tr(self.name())

    def tr(self, string):
        return QCoreApplication.translate('Processing', string)

    def createInstance(self):
        return ResetForstSettings()

    def createCustomParametersWidget(self, parent):
        self.parent = parent
        return self.dlg

    def svgIconPath(self):
        return os.path.abspath("./icons/reset_settings.png")

    def icon(self):
        return QIcon(":/plugins/icons/reset_settings.png")

    def reject(self):
        self.dlg.close()

    def unload(self):
        if QgsSettings().value("resetForstSettings/mode") == 'yes':
            ResetForstSettings().copy_settings()
            QgsSettings().setValue("resetForstSettings/mode", 'no')

    def copy_settings(self):
        current_profile = QgsApplication.qgisSettingsDirPath()
        copy_file(os.path.join(current_profile, "python/plugins/forst_processing/_7_reset/default_settings/QGIS3.ini"),
                  os.path.join(current_profile, "QGIS/QGIS3.ini"))
        copy_file(os.path.join(current_profile,
                               "python/plugins/forst_processing/_7_reset/default_settings/QGISCUSTOMIZATION3.ini"),
                  os.path.join(current_profile, "QGIS/QGISCUSTOMIZATION3.ini"))
        #copy_file(os.path.join(current_profile,
        #                       "python/plugins/forstprocessing/_7_reset/default_settings/qgis.db"),
        #          os.path.join(current_profile, "qgis.db"))
        #copy_file(os.path.join(current_profile,
        #                       "python/plugins/forstprocessing/_7_reset/default_settings/qgis-auth.db"),
        #          os.path.join(current_profile, "qgis-auth.db"))
        #copy_file(os.path.join(current_profile,
        #                       "python/plugins/forstprocessing/_7_reset/default_settings/symbology-style.db"),
        #          os.path.join(current_profile, "symbology-style.db"))
        QMessageBox.information(iface.mainWindow(), "Information", u"Einstellung zurückgestellt.")

        return 0

