# -*- coding: utf-8 -*-
"""
/***************************************************************************
 forst_wms
                                 A QGIS plugin
 Öffnet WMS von basemap.at
                              -------------------
        begin                : 2015-01-19
        git sha              : $Format:%H$
        copyright            : (C) 2015 by Christina Ludwig, TU Wien
        email                : christina_ludwig@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from qgis.PyQt.QtCore import QSettings, QTranslator, qVersion, QCoreApplication
from qgis.PyQt.QtWidgets import QAction
# Initialize Qt resources from file resources.py
#import resources_rc
# Import the code for the dialog
from .forstwms_dialog import ForstWMSDialog
import os.path


class ForstWMS:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'forstwms_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        # Create the dialog (after translation) and keep reference
        self.dlg = ForstWMSDialog()

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&QGIS Forst WMS Dienste')

        # TODO: We are going to let the user set this up in a future iteration
        self.toolbar = self.iface.addToolBar(u'QGIS Forst WMS Dienste')
        self.toolbar.setObjectName(u'QGIS Forst WMS Dienste')

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('QGIS Forst WMS Dienste', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        menu,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=False,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        #icon = QIcon(icon_path)
        action = QAction(text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            self.toolbar.addAction(action)

        if add_to_menu:
            self.iface.addPluginToWebMenu(
                menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""
        icon_path = ':/plugins/forst_wms/icon.png'

        self.add_action(
            icon_path,
            text=self.tr(u'Geoland Basemap High DPI'),
            callback=self.run_geoland_high_dpi,
            menu=self.menu,
            parent=self.iface.mainWindow())

        self.add_action(
            icon_path,
            text=self.tr(u'Geoland Basemap'),
            callback=self.run_geoland_basemap,
            menu=self.menu,
            parent=self.iface.mainWindow()),

        self.add_action(
            icon_path,
            text=self.tr(u'Geoland Basemap Gelände'),
            callback=self.run_geoland_gelaende,
            menu=self.menu,
            parent=self.iface.mainWindow()),

        self.add_action(
            icon_path,
            text=self.tr(u'Geoland Basemap Grau'),
            callback=self.run_geoland_grau,
            menu=self.menu,
            parent=self.iface.mainWindow())

        self.add_action(
            icon_path,
            text=self.tr(u'Geoland Basemap Oberfläche'),
            callback=self.run_geoland_oberflaeche,
            menu=self.menu,
            parent=self.iface.mainWindow()),

        self.add_action(
            icon_path,
            text=self.tr(u'Geoland Basemap Orthofoto'),
            callback=self.run_geoland_orthofoto,
            menu=self.menu,
            parent=self.iface.mainWindow()),

        self.add_action(
            icon_path,
            text=self.tr(u'Geoland Basemap Overlay'),
            callback=self.run_geoland_overlay,
            menu=self.menu,
            parent=self.iface.mainWindow())

        self.add_action(
            icon_path,
            text=self.tr(u'Bing Aerial'),
            callback=self.run_bing,
            menu=self.menu,
            parent=self.iface.mainWindow())

        self.add_action(
            icon_path,
            text=self.tr(u'Google Satellite'),
            callback=self.run_google,
            menu=self.menu,
            parent=self.iface.mainWindow())

        self.add_action(
            icon_path,
            text=self.tr(u'OpenStreetMap'),
            callback=self.run_osm,
            menu=self.menu,
            parent=self.iface.mainWindow())


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginWebMenu(
                self.tr(u'&QGIS Forst WMS Dienste'),
                action)
            self.iface.removeToolBarIcon(action)


    def run_geoland_high_dpi(self):
        self.iface.addRasterLayer("crs=EPSG:3857&dpiMode=7&featureCount=10&format=image/jpeg&layers=bmaphidpi&styles=normal&tileMatrixSet=google3857&url=http://www.basemap.at/wmts/1.0.0/WMTSCapabilities.xml", "Basemap High DPI", "wms")

    def run_geoland_basemap(self):
        self.iface.addRasterLayer("crs=EPSG:3857&dpiMode=7&format=image/png&layers=geolandbasemap&styles=normal&tileMatrixSet=google3857&url=https://www.basemap.at/wmts/1.0.0/WMTSCapabilities.xml", "Geoland Basemap", "wms")

    def run_geoland_gelaende(self):
        self.iface.addRasterLayer("crs=EPSG:3857&dpiMode=7&format=image/jpeg&layers=bmapgelaende&styles=grau&tileMatrixSet=google3857&url=https://www.basemap.at/wmts/1.0.0/WMTSCapabilities.xml", "Geoland Basemap Gelände", "wms")

    def run_geoland_grau(self):
        self.iface.addRasterLayer("crs=EPSG:3857&dpiMode=7&featureCount=10&format=image/png&layers=bmapgrau&styles=normal&tileMatrixSet=google3857&url=http://www.basemap.at/wmts/1.0.0/WMTSCapabilities.xml", "Geoland Basemap Grau", "wms")

    def run_geoland_oberflaeche(self):
        self.iface.addRasterLayer("crs=EPSG:3857&dpiMode=7&format=image/jpeg&layers=bmapoberflaeche&styles=grau&tileMatrixSet=google3857&url=https://www.basemap.at/wmts/1.0.0/WMTSCapabilities.xml", "Geoland Basemap Oberfläche", "wms")

    def run_geoland_orthofoto(self):
        self.iface.addRasterLayer("crs=EPSG:3857&dpiMode=7&format=image/jpeg&layers=bmaporthofoto30cm&styles=normal&tileMatrixSet=google3857&url=https://www.basemap.at/wmts/1.0.0/WMTSCapabilities.xml", "Geoland Basemap Orthofoto", "wms")

    def run_geoland_overlay(self):
        self.iface.addRasterLayer("crs=EPSG:3857&dpiMode=7&format=image/png&layers=bmapoverlay&styles=normal&tileMatrixSet=google3857&url=https://www.basemap.at/wmts/1.0.0/WMTSCapabilities.xml", "Geoland Basemap Overlay", "wms")

    def run_bing(self):
        self.iface.addRasterLayer("type=xyz&url=http://ecn.t3.tiles.virtualearth.net/tiles/a{q}.jpeg?g=1", "Bing Aerial", "wms")

    def run_osm(self):
        self.iface.addRasterLayer("type=xyz&url=http://tile.openstreetmap.org/%7Bz%7D/%7Bx%7D/%7By%7D.png&zmax=18&zmin=0", "OpenStreetMap", "wms")

    def run_google(self):
        self.iface.addRasterLayer("type=xyz&url=http://www.google.cn/maps/vt?lyrs%3Ds@189%26gl%3Dcn%26x%3D%7Bx%7D%26y%3D%7By%7D%26z%3D%7Bz%7D&zmax=18&zmin=0", "Google Satellite", "wms")

